package org.bouil.twifip;

public class TrackInfo {

    public String title;
    public String artist;
    public String album;

    public static TrackInfo createTrackInfo(String artist, String album, String title) {
        if (title.contains("BONNE NUIT SUR FIP") || title.startsWith("FIP ACTUALITE") ||
                (artist == null && album == null)) {
            return null;
        }

        TrackInfo trackInfo = new TrackInfo();
        trackInfo.title = title;
        trackInfo.artist = artist;
        trackInfo.album = album;

        return trackInfo;
    }

    public static TrackInfo createTrackInfo(String current) {
        String[] split = current.trim().split(" - ");
        if (split.length >= 3) {
            String title = split[0].trim();
            String artist = split[1].trim();
            String album = split[2].trim();

            if (title.contains("BONNE NUIT SUR FIP") || title.startsWith("FIP ACTUALITE") ||
                    (artist == null && album == null)) {
                return null;
            }

            TrackInfo trackInfo = new TrackInfo();
            trackInfo.title = title;
            trackInfo.artist = artist;
            trackInfo.album = album;
            trackInfo.correctAlbum();

            return trackInfo;
        }
        return null;
    }

    private void correctAlbum() {
        int lastSpacePosition = album.lastIndexOf(' ');
        if (lastSpacePosition != -1) {
            String year = album.substring(lastSpacePosition + 1, album.length()).trim();
            try {
                Integer.parseInt(year);
                album = album.substring(0, lastSpacePosition).trim();
            } catch (NumberFormatException e) {
                // no year
            }
        }
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof TrackInfo)) {
            return false;
        }

        TrackInfo trackInfo = (TrackInfo) o;

        if (album != null ? !album.equals(trackInfo.album) : trackInfo.album != null) {
            return false;
        }
        if (artist != null ? !artist.equals(trackInfo.artist) : trackInfo.artist != null) {
            return false;
        }
        if (title != null ? !title.equals(trackInfo.title) : trackInfo.title != null) {
            return false;
        }

        return true;
    }

    @Override
    public int hashCode() {
        int result = title != null ? title.hashCode() : 0;
        result = 31 * result + (artist != null ? artist.hashCode() : 0);
        result = 31 * result + (album != null ? album.hashCode() : 0);
        return result;
    }

    @Override
    public String toString() {
        return "TrackInfo{" + "title='" + title + '\'' + ", artist='" + artist + '\'' + ", album='" + album + '\'' +
                '}';
    }
}
