package org.bouil.twifip;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.Date;
import java.util.logging.Logger;

@Component
public class Twifip {

    private static final Logger log = Logger.getLogger(Twifip.class.getName());

    @Autowired
    private TwifipUtils twifipUtils;

    private TrackInfo previous = null;

    private Date lastUpdateDate = null;

    private static final int SECOND = 1000;
    private static final int MINUTE = 60 * SECOND;
    private static final int HOUR = 60 * MINUTE;
    private static final int DAY = 24 * HOUR;

    @Scheduled(fixedDelay = 15 * SECOND) // every day
    public void execute() throws IOException {
        synchronized (this) {

            TrackInfo current = twifipUtils.getCurrentFip();
            if (current != null) {
                if (!current.equals(previous)) {
                    log.info("Updating to " + current);
                    twifipUtils.updateLastfm(current);
                    // twifipUtils.updateTwitter(current);
                    previous = current;
                    lastUpdateDate = new Date();
                } else {
                    log.info("same as previous " + current);
                }
            } else {
                log.info("null in fip html");
            }
        }
    }

    public TrackInfo getPrevious() {
        return previous;
    }

    public Date getLastUpdateDate() {
        return lastUpdateDate;
    }
}


