package org.bouil.twifip;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;
import java.util.Random;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import de.umass.lastfm.Authenticator;
import de.umass.lastfm.Caller;
import de.umass.lastfm.Result;
import de.umass.lastfm.Session;
import de.umass.lastfm.Track;
import de.umass.lastfm.scrobble.ScrobbleData;
import de.umass.lastfm.scrobble.ScrobbleResult;
import twitter4j.GeoLocation;
import twitter4j.ResponseList;
import twitter4j.Status;
import twitter4j.StatusUpdate;
import twitter4j.Twitter;
import twitter4j.TwitterFactory;
import twitter4j.internal.org.json.JSONException;
import twitter4j.internal.org.json.JSONObject;

@Component
public class TwifipUtils {

    private static org.slf4j.Logger log = LoggerFactory.getLogger(TwifipUtils.class);

    private static TwitterFactory twitterFactory = new TwitterFactory();

    private static String lastFmUser;

    private static String lastFmPassword;

    private static String lastfmApiKey;

    private static String lastFmApiSecret;

    private static DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();

    private static XPathFactory xPathFactory = XPathFactory.newInstance();

    public TwifipUtils() {
        Properties p = new Properties();
        try {
            p.load(getClass().getResourceAsStream("/lastfm.properties"));
            lastFmUser = p.getProperty("login");
            lastFmPassword = p.getProperty("password");
            lastfmApiKey = p.getProperty("apiKey");
            lastFmApiSecret = p.getProperty("apiSecret");

            Caller.getInstance().setCache(null);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }
    }

    public void updateLastfm(TrackInfo trackInfo) {
        try {
            if (trackInfo != null) {
                Session session =
                        Authenticator.getMobileSession(lastFmUser, lastFmPassword, lastfmApiKey, lastFmApiSecret);

                ScrobbleData scrobbleData = new ScrobbleData();
                scrobbleData.setAlbum(trackInfo.album);
                scrobbleData.setArtist(trackInfo.artist);
                scrobbleData.setTrack(trackInfo.title);
                scrobbleData.setTimestamp((int) (System.currentTimeMillis() / 1000));

                ScrobbleResult result = Track.scrobble(scrobbleData, session);
                log.debug(result.toString());

                Result tagResult = Track.addTags(trackInfo.artist, trackInfo.title, "fip", session);
                log.debug(result.toString());
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    public boolean updateTwitter(TrackInfo current) {
        try {
            String newStatus = current.title + " - " + current.artist + " - " + current.album;
            if (newStatus.length() > 140) {
                newStatus = current.title + " - " + current.artist;
            }

            Twitter twitter = twitterFactory.getInstance();

            ResponseList<Status> userTimeline = twitter.getUserTimeline();
            if (userTimeline.size() > 0) {
                Status status = userTimeline.get(0);
                if (StringUtils.endsWithIgnoreCase(newStatus.trim(), status.getText().trim())) {
                    log.debug("Status is unchanged");
                    return false;
                }
            }
            StatusUpdate statusUpdate = new StatusUpdate(newStatus);
            // Maison de la Radio - Paris
            statusUpdate.setLocation(new GeoLocation(48.85253, 2.27813));

            Status status = twitter.updateStatus(statusUpdate);
            log.debug("Twitter status id " + status.getId());
            return true;
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return false;
        }
    }

    public TrackInfo getCurrentFip() {

        try {
            String urlString =
                    "http://www.fipradio.fr/sites/default/files/direct-large.json?_=" + System.currentTimeMillis();
            log.debug("Loading " + urlString);
            URL url = new URL(urlString);
            BufferedReader reader = new BufferedReader(new InputStreamReader(url.openStream(), "ISO-8859-15"));
            String buf = null;
            StringBuilder sb = new StringBuilder();
            while ((buf = reader.readLine()) != null) {
                sb.append(buf);
            }
            reader.close();

            String html = "<html>" + decodeJson(sb.toString()) + "</html>";

            html = html.replaceAll("& ", "&amp; ");

            InputStream in = new ByteArrayInputStream(html.getBytes("UTF-8"));
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document document = documentBuilder.parse(in);

            XPath xpath = xPathFactory.newXPath();
            XPathExpression artistXPath = xpath.compile(
                    "/html/div[@id=\"direct-list\"]/div[@class=\"direct-current\"]//div[@class=\"inner\"]/div[@class=\"artiste\"]");
            String artiste = (String) artistXPath.evaluate(document, XPathConstants.STRING);

            XPathExpression titreXPath = xpath.compile(
                    "/html/div[@id=\"direct-list\"]/div[@class=\"direct-current\"]//div[@class=\"inner\"]/div[@class=\"titre\"]");
            String titre = (String) titreXPath.evaluate(document, XPathConstants.STRING);

            XPathExpression albumXPath = xpath.compile(
                    "/html/div[@id=\"direct-list\"]/div[@class=\"direct-current\"]//div[@class=\"inner\"]/div[@class=\"album\"]");
            String album = (String) albumXPath.evaluate(document, XPathConstants.STRING);

            return TrackInfo.createTrackInfo(artiste, album, titre);

        } catch (MalformedURLException e) {
            log.error(e.getMessage(), e);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        } catch (JSONException e) {
            log.error(e.getMessage(), e);
        } catch (SAXException e) {
            log.error(e.getMessage(), e);
        } catch (XPathExpressionException e) {
            log.error(e.getMessage(), e);
        } catch (ParserConfigurationException e) {
            log.error(e.getMessage(), e);
        }
        return null;
    }

    public TrackInfo getMockedCurrentFip() {
        Random r = new Random(System.currentTimeMillis());
        int n = 100000;
        String artiste = Integer.toString(r.nextInt(n));
        String titre = Integer.toString(r.nextInt(n));
        String album = Integer.toString(r.nextInt(n));
        return TrackInfo.createTrackInfo(artiste, album, titre);
    }

    private String decodeJson(String json) throws JSONException {
        JSONObject jsonObject = new JSONObject(json);
        return jsonObject.getString("html");
    }

    /**
     * Test method
     *
     * @param args
     */
    public static void main(String[] args) {
        /* display current fip */
        //        final TwifipUtils twifipUtils = new TwifipUtils();
        //        TrackInfo currentFip = twifipUtils.getCurrentFip();
        //        System.out.println(currentFip);
        //        System.out.println(twifipUtils.updateTwitter(currentFip));
        //        twifipUtils.updateLastfm(currentFip);
    }
}

