// Copyright 2011 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.bouil.twifip.tapestry.components;

import java.util.List;

import org.apache.tapestry5.BindingConstants;
import org.apache.tapestry5.ClientElement;
import org.apache.tapestry5.ComponentResources;
import org.apache.tapestry5.MarkupWriter;
import org.apache.tapestry5.alerts.Alert;
import org.apache.tapestry5.alerts.AlertStorage;
import org.apache.tapestry5.annotations.Environmental;
import org.apache.tapestry5.annotations.Parameter;
import org.apache.tapestry5.annotations.SessionState;
import org.apache.tapestry5.annotations.SupportsInformalParameters;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.services.javascript.JavaScriptSupport;

/**
 * Renders out an empty {@code <div>} element and provides JavaScript initialization to make the element
 * the container for alerts. After rendering markup (and initialization JavaScript), it
 * {@linkplain org.apache.tapestry5.alerts.AlertStorage#dismissNonPersistent() removes all non-persistent alerts}.
 *
 * @tapestrydoc
 * @since 5.3
 */
@SupportsInformalParameters
public class Alerts implements ClientElement {

    @Parameter(value = "×", defaultPrefix = BindingConstants.LITERAL)
    private String dismissText;

    @Inject
    private ComponentResources resources;

    @Environmental
    private JavaScriptSupport javaScriptSupport;

    @SessionState(create = false)
    private AlertStorage storage;

    private String clientId;

    public String getClientId() {
        return clientId;
    }

    boolean beginRender(MarkupWriter writer) {
        clientId = javaScriptSupport.allocateClientId(resources);

        if (storage != null) {
            List<Alert> alerts = storage.getAlerts();
            if (!alerts.isEmpty()) {
                for (Alert alert : alerts) {

                    String alertType = alert.severity.name().toLowerCase();
                    String css = alertType;
                    if ("info".equals(alertType)) {
                        css = "success";
                    }
                    css = "alert-" + css;

                    writer.element("div", "id", clientId, "class", "alert " + css);
                    {
                        resources.renderInformalParameters(writer);

                        String dismiss = resources.createEventLink("dismiss", alert.id).toURI();
                        if (alert.duration.persistent) {
                            writer.element("a", "href", dismiss, "class", "close");

                        } else {
                            writer.element("a", "href", "#", "class", "close", "data-dismiss", "alert");
                        }
                        writer.write(dismissText);
                        writer.end();

                        writer.write(alert.message);

                    }
                    writer.end();
                }

            }
            storage.dismissNonPersistent();
        }

        return false;
    }

    Object onDismiss(Long alertId) {
        // If the alert was created inside an Ajax request and AlertStorage did not previously
        // exist, it can be null when the dismiss event comes up from the client.
        if (storage != null) {
            if (alertId != null) {
                storage.dismiss(alertId);
            } else {
                storage.dismissAll();
            }
        }

        return null;
    }
}
