var twifip = angular.module("twifip", []);

twifip.controller("TwifipCtrl", function ($scope, $timeout, $http) {

    var refreshRate = 5000;
    var maxHistoryCount = 100;
    var defaultTitle = "What's now playing on FIP Radio";

    var img = angular.element("#desktopNotificationTitle").data("img");

    $scope.desktopNotifications = false;
    $scope.desktopNotificationsPermissionsGranted = false;

    $scope.titleHistory = [];

    $scope.fetchTitle = function () {
        if ($scope.titleStream) {
            $http.get('/json').success(function (trackInfo) {
                $scope.htmlTitle = trackInfo.title + " - " + trackInfo.artist + " [" + defaultTitle + "]";
                var previousTrack = null;
                if ($scope.titleHistory.length > 0) {
                    previousTrack = $scope.titleHistory[0];
                }
                if (!angular.equals(trackInfo, previousTrack)) {

                    $scope.titleHistory.unshift(trackInfo);

                    if ($scope.titleHistory.length > maxHistoryCount) {
                        $scope.titleHistory.pop();
                    }

                    if ($scope.desktopNotifications && $scope.desktopNotificationsPermissionsGranted) {
                        var popup = window.webkitNotifications.createNotification(img, trackInfo.title,
                                                                                  trackInfo.artist + " - " +
                                                                                      trackInfo.album);
                        // Click event on notification
                        popup.onclick = function () {
                            // Focus on web-app
                            window.focus();
                            // Remove notification
                            this.cancel();

                        };
                        // hide popup after a few sec
                        popup.ondisplay = function(event) {
                            $timeout(function() {
                                event.currentTarget.cancel();
                            }, 3000, false);
                        };
                        popup.show();
                    }
                }
            });
            $timeout($scope.fetchTitle, refreshRate);
        } else {
            $scope.htmlTitle = defaultTitle;
        }
    }

    $scope.cssClassButton = function (buttonType, toggle) {
        if (buttonType == "on" && toggle) {
            return " btn-success";
        }
        if (buttonType == "off" && !toggle) {
            return " btn-danger";
        }
        return "";
    }

    $scope.toggleNotifications = function () {
        if (!$scope.desktopNotificationsPermissionsGranted) {
            var checkPermission = window.webkitNotifications.checkPermission();
            if (checkPermission == 0) {
                $scope.desktopNotificationsPermissionsGranted = true;
            } else {
                window.webkitNotifications.requestPermission(function () {
                    $scope.desktopNotificationsPermissionsGranted = true;
                });
            }
        }

    };

    $scope.$watch("titleStream", $scope.fetchTitle);
    $scope.$watch("desktopNotifications", $scope.toggleNotifications);
    $scope.titleStream = true;

});
